########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/o_agedtl.4gs/lowlevel.4gl  1.10  Delta: 2/4/91
########################################################################

globals "globals.4gl"

#######################################################################
function before_group(group_key)
#######################################################################
# group_key is
#     "first_row" when the first row of the report is being processed,
#     report_sort (vend_code or bus_name) for vendor-level breaks,
#     "inv_doc_no" for inv_doc_no-level breaks.
#
    define
        group_key   char(20)    # group identification
    # Trap fatal errors
    whenever error call error_handler
    if group_key = "first_row" then
        # insert "on first row" code here
        return
    end if
    if group_key = report_sort then
        # insert "before group of report_sort" code here
        let rpt.vend_code = curs.vend_code
        let rpt.bus_name = curs.bus_name
        case
            when report_sort = "bus_name, vend_code"
                 let rpt.order_group = rpt.bus_name,",",curs.vend_code
            otherwise
                 let rpt.order_group = curs.vend_code
        end case
        let rpt.sub_total = 0
        let rpt.sub_per1 = 0
        let rpt.sub_per2 = 0
        let rpt.sub_per3 = 0
        let rpt.sub_per4 = 0
        return
    end if
    if group_key = "inv_doc_no" then
        # insert "before group of inv_doc_no" code here
        let rpt.out_date = curs.date_used
        let rpt.per1 = null
        let rpt.per2 = null
        let rpt.per3 = null
        let rpt.per4 = null
        let rpt.balance = 0
        let rpt.inv_doc_no = curs.inv_doc_no
	    let rpt.inv_no = null
        # inv_doc_no = 0 for all on-account amounts. They have no entry
        # in stpopend.  Don't attempt the select for on-account amounts.
		if curs.inv_doc_no != 0
		then
            select stpopend.inv_no
            into   rpt.inv_no
            from   stpopend
            where  stpopend.vend_code = curs.vend_code and
                   stpopend.doc_no = curs.inv_doc_no
		end if
        return
    end if
end function
# before_group()
#######################################################################
function on_every_row()
#######################################################################
# This function accumulates rpt.balance for on account amounts.  The 
# original "select into temp" statement groups all stractvd rows that 
# pertain to a single invoice where the aggregate total is not equal to 
# zero.  The exception is on-account amounts - these amounts are not 
# grouped and totaled; because the date_used column for these on account 
# rows is null, these on account rows cannot be totaled. The purpose of 
# accumulating a rpt.balance in this function is to accumulate a total 
# for on-account amounts for a single vendor.
#
    let rpt.balance = rpt.balance + curs.balance
    return
end function
# on_every_row()
#######################################################################
function after_group(group_key)
#######################################################################
# group_key is either "vend_code" for vendor-level breaks, 
#                     "inv_doc_no" for inv_doc_no level breaks, or
#                     "last_row" when last row of report is processed.
# If group_key is "inv_doc_no", then this function prepares the detail
# line for printing.
#
    define
        group_key   char(20)    # group identification
    if group_key = report_sort then
        # insert "after group of report_sort" code here
        # calculate total for each vendor (rpt.sub_total)
        let rpt.sub_total = rpt.sub_per1 + rpt.sub_per2 + 
                            rpt.sub_per3 + rpt.sub_per4
        # accumulate aging period totals for entire report
        let rpt.tot_per1 = rpt.tot_per1 + rpt.sub_per1
        let rpt.tot_per2 = rpt.tot_per2 + rpt.sub_per2
        let rpt.tot_per3 = rpt.tot_per3 + rpt.sub_per3
        let rpt.tot_per4 = rpt.tot_per4 + rpt.sub_per4
        return
    end if
    if group_key = "inv_doc_no" then
      # insert "after group of inv_doc_no" code here
      if is_aging_rpt
      then    # aging report processing
        case
            # on-account amounts are identified by rpt.inv_doc_no=0
            when rpt.inv_doc_no = 0
                let rpt.inv_no = "On Account"
                let rpt.out_date = " "  # no date for on-account amounts
                let rpt.per1 = rpt.balance
                # if on_account rows total up to 0, do not print line
                if rpt.per1 = 0 then let rpt.per1 = null end if
                let rpt.sub_per1 = rpt.sub_per1 + rpt.balance
                exit case
            when rpt.aging_date - rpt.out_date <= m_pcntrc.age_per1
                let rpt.per1 = rpt.balance
                let rpt.sub_per1 = rpt.sub_per1 + rpt.balance
                exit case
            when rpt.aging_date - rpt.out_date <= m_pcntrc.age_per2
                let rpt.per2 = rpt.balance
                let rpt.sub_per2 = rpt.sub_per2 + rpt.balance
                exit case
            when rpt.aging_date - rpt.out_date <= m_pcntrc.age_per3
                let rpt.per3 = rpt.balance
                let rpt.sub_per3 = rpt.sub_per3 + rpt.balance
                exit case
            otherwise
                let rpt.per4 = rpt.balance
                let rpt.sub_per4 = rpt.sub_per4 + rpt.balance
                exit case
        end case
      else  # cash requirement report processing
        case
            # on-account amounts are identified by rpt.inv_doc_no=0
            when rpt.inv_doc_no = 0
                let rpt.inv_no = "On Account"
                let rpt.out_date = " "  # no date for on-account amounts
                let rpt.per1 = rpt.balance
                # if on_account rows total up to 0, do not print line
                if rpt.per1 = 0 then let rpt.per1 = null end if
                let rpt.sub_per1 = rpt.sub_per1 + rpt.balance
                exit case
            when rpt.out_date - rpt.aging_date <= m_pcntrc.age_per1
                let rpt.per1 = rpt.balance
                let rpt.sub_per1 = rpt.sub_per1 + rpt.balance
                exit case
            when rpt.out_date - rpt.aging_date <= m_pcntrc.age_per2
                let rpt.per2 = rpt.balance
                let rpt.sub_per2 = rpt.sub_per2 + rpt.balance
                exit case
            when rpt.out_date - rpt.aging_date <= m_pcntrc.age_per3
                let rpt.per3 = rpt.balance
                let rpt.sub_per3 = rpt.sub_per3 + rpt.balance
                exit case
            otherwise
                let rpt.per4 = rpt.balance
                let rpt.sub_per4 = rpt.sub_per4 + rpt.balance
                exit case
        end case
      end if
      return
    end if
    if group_key = "last_row" then
        # insert "on last row" code here
        let rpt.tot_total = rpt.tot_per1 + rpt.tot_per2 + 
                            rpt.tot_per3 + rpt.tot_per4
        return
    end if
end function
# after_group()
