########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/o_chkreg.4gs/lowlevel.4gl  1.10  Delta: 1/4/91
########################################################################

globals "globals.4gl"

#######################################################################
function before_group(group_key)
#######################################################################
#
    define
        group_key   char(20),   # group identification
        sql_str char(128)

    # Trap fatal errors
    whenever error call error_handler

    if group_key = "first_row" then
        # insert "on first row" code here
        let sql_str = "select balance, activity, this_month from stxchrtd ",
                        "where acct_no = ? and department = ? and ",
                           "period_month = ? and period_year = ?"
        prepare get_bal from sql_str
        declare bal_curs cursor for get_bal
        return
    end if

    if group_key = "acct_no" then
        # insert "before group of acct_no" code here
        return
    end if

    if group_key = "department" then
        # insert "before group of department" code here
        call get_beg_bal() returning rpt.beg_amount
        let rpt.bal_amount = rpt.beg_amount
        let rec_flag = false
        return
    end if

    if group_key = "doc_date" then
        # insert "before group of doc_date" code here
        return
    end if

    if group_key = "orig_journal" then
        # insert "before group of orig_journal" code here
        return
    end if

    if group_key = "inv_chk_no" then
        # insert "before group of inv_chk_no" code here
        return
    end if

end function
# before_group()

#######################################################################
function on_every_row()
#######################################################################
#  this function prepares the report record from the cursor record
#  and other data.
#
    let rpt.doc_no = curs.doc_no
    let rpt.incr_with_crdt = curs.incr_with_crdt
    let rpt.debit_credit = curs.debit_credit
    let rpt.amount = curs.amount
    let rpt.acct_no = curs.acct_no
    let rpt.department = curs.department
    let rpt.acct_desc = curs.acct_desc
    let rpt.doc_date = curs.doc_date
    let rpt.inv_chk_no = curs.inv_chk_no
    let rpt.orig_journal = curs.orig_journal
    let rpt.ref_code = curs.ref_code
    let rpt.doc_desc = curs.doc_desc
    if curs.incr_with_crdt != "Y"  # not increase with a credit
    then
      if curs.debit_credit = "D" 
      then 
        let rpt.line_amount = rpt.amount 
      else 
        let rpt.line_amount = rpt.amount * -1
      end if
    else
      if curs.debit_credit = "D" 
      then 
        let rpt.line_amount = rpt.amount * -1 
      else 
        let rpt.line_amount = rpt.amount
      end if
    end if
    let rpt.bal_amount = rpt.bal_amount + rpt.line_amount
    let rpt.reconciled = curs.reconciled
    if rpt.reconciled = "Y"
    then
        let rpt.asterisk = "*"
        let rec_flag = true
    else
        let rpt.asterisk = null
    end if
    return
end function
# on_every_row()

#######################################################################
function after_group(group_key)
#######################################################################
#
    define
        group_key   char(20)    # group identification

    if group_key = "acct_no" then
        # insert "after group of acct_no" code here
        return
    end if

    if group_key = "department" then
        # insert "after group of department" code here
        if rec_flag
        then
            let rpt.rec_message = 
                    "[*] Indicates that this check has been reconciled."
        else
            let rpt.rec_message = null
        end if
        return
    end if

    if group_key = "doc_date" then
        # insert "after group of doc_date" code here
        return
    end if

    if group_key = "orig_journal" then
        # insert "after group of orig_journal" code here
        return
    end if

    if group_key = "inv_chk_no" then
        # insert "after group of inv_chk_no" code here
        return
    end if

    if group_key = "last_row" then
        # insert "on last row" code here
        return
    end if

end function
# after_group()

#######################################################################
function get_beg_bal()
# returning the begining balance amount for the given cash account
#######################################################################
#
# if g/l is installed this function will calculate the beginning balance 
# for the given cash account/department on a given date.
#
    define
        trx_per char(2),
        trx_per_year char(4),
        trx_start_date date,
        db_amount decimal(12),
        cr_amount decimal(12),
        trx_bal decimal(12),
        trx_activity decimal(12),
        trx_this_month decimal(12),
        start_bal decimal(12)

    # if gl is not installed return balance of 0
    if not table_exist("stgcntrc") then return 0 end if

    # determine which period/year of the start date
    select period, period_year into trx_per, trx_per_year from stxperdr
          where start_date <= rpt.start_date and end_date >= rpt.start_date
          and period != "00"

    if sqlca.sqlcode = notfound
    then
        return 0
    end if

    # get the ending balance for the given account
    open bal_curs using curs.acct_no, curs.department, trx_per, trx_per_year
    fetch bal_curs into trx_bal, trx_activity, trx_this_month
    close bal_curs

    if trx_bal is null 
    then 
        return 0 
    end if

    if trx_activity is null 
    then 
        let trx_activity = 0 
    end if

    if trx_this_month is null 
    then 
        let trx_this_month = 0
    end if

    let trx_bal = trx_bal - trx_activity - trx_this_month

    # get the ending date for the given period/year
    select start_date into trx_start_date
      from stxperdr where period = trx_per and period_year = trx_per_year

    # select all debit amounts between rpt.start_date and trx_end_date
    select sum(amount) into db_amount from stxckrgd, stxtranr 
      where stxckrgd.acct_no = curs.acct_no and
            stxckrgd.department = curs.department and
            stxckrgd.orig_journal = stxtranr.orig_journal and 
            stxckrgd.doc_no = stxtranr.doc_no and 
            stxtranr.doc_date < rpt.start_date and 
            stxtranr.doc_date >= trx_start_date and
            stxckrgd.debit_credit = "D"

    if db_amount is null then let db_amount = 0 end if

    # select all credit amounts between rpt.start_date and trx_end_date
    select sum(amount) into cr_amount from stxckrgd, stxtranr 
      where stxckrgd.acct_no = curs.acct_no and
            stxckrgd.department = curs.department and
            stxckrgd.orig_journal = stxtranr.orig_journal and 
            stxckrgd.doc_no = stxtranr.doc_no and 
            stxtranr.doc_date < rpt.start_date and 
            stxtranr.doc_date >= trx_start_date and
            stxckrgd.debit_credit = "C"

    if cr_amount is null then let cr_amount = 0 end if

    if curs.incr_with_crdt = "Y"  # increase with a credit
    then
        # subtract all db amounts and add all cr amounts
        let start_bal = trx_bal - db_amount + cr_amount
    else
        # subtract all cr amounts and add all db amounts
        let start_bal = trx_bal - cr_amount + db_amount
    end if

    return start_bal

end function 
# get_beg_bal()
