########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../ap.4gm/o_chkreg.4gs/midlevel.4gl  1.7  Delta: 2/4/91
########################################################################

globals "globals.4gl"

#######################################################################
function ml_join()
#######################################################################
#
    # Trap fatal errors
    whenever error call error_handler

    let sel_join =
        "stxckrgr.acct_no = stxckrgd.acct_no and ",
        "stxckrgr.department = stxckrgd.department and ",
        "stxckrgd.orig_journal = stxtranr.orig_journal and ",
        "stxckrgd.doc_no = stxtranr.doc_no and ",
        "stxckrgr.acct_no = stxchrtr.acct_no"
    return
end function
# ml_join()

#######################################################################
function ml_filter()
#######################################################################
#
    define quote char(1),
           scratch char(200)

    let quote = "\""
    call dat_rnge() returning rpt.start_date, rpt.end_date
    let ct.sel_filter = "stxtranr.doc_date >= ", quote, rpt.start_date, quote,
                   " and stxtranr.doc_date <= ", quote, rpt.end_date, quote
    
    call sel_cash_acct() returning scratch

    let ct.sel_filter = ct.sel_filter clipped, " and ", scratch clipped

    return
end function
# ml_filter()

#######################################################################
function ml_order()
#######################################################################
#
    let ct.sel_order = "stxckrgr.acct_no, ",
                       "stxckrgr.department, ",
                       "stxtranr.doc_date, ",
                       "stxckrgd.orig_journal, ",
                       "stxckrgd.inv_chk_no"
    return
end function
# ml_order()

#######################################################################
function ml_getcount()
#######################################################################
#
    let ct.sel_stmt =
        "select count(*) ",
        "from stxckrgr, stxckrgd, stxtranr, outer stxchrtr ",
        "where (", sel_join clipped,
        ") and (", ct.sel_filter clipped, ")"
    prepare get_count from ct.sel_stmt
    declare count_cursor cursor for get_count
    open count_cursor
    fetch count_cursor into ct.num_rows
    close count_cursor
    return
end function
# ml_getcount()

#######################################################################
function ml_define_cur()
#######################################################################
#
    let ct.sel_stmt =
        "select ",
            "stxckrgd.doc_no, ",
            "stxchrtr.incr_with_crdt, ",
            "stxckrgd.debit_credit, ",
            "stxckrgd.amount, ",
            "stxckrgr.acct_no, ",
            "stxckrgr.department, ",
            "stxchrtr.acct_desc, ",
            "stxtranr.doc_date, ",
            "stxckrgd.inv_chk_no, ",
            "stxckrgd.orig_journal, ",
            "stxtranr.ref_code, ",
            "stxtranr.doc_desc, ",
            "stxckrgd.reconciled ",
        "from stxckrgr, stxckrgd, stxtranr, outer stxchrtr ",
        "where (", sel_join clipped,
        ") and (", ct.sel_filter clipped, ")"
    if ct.sel_order is not null
    then
        let ct.sel_stmt = ct.sel_stmt clipped,
        " order by ", ct.sel_order clipped
    end if
    prepare get_curs from ct.sel_stmt
    declare rpt_cursor cursor for get_curs
    open rpt_cursor
    return
end function
# ml_define_cur()

#######################################################################
function ml_fetch()
#######################################################################
#
    fetch rpt_cursor into curs_next.*
    return
end function
# ml_fetch()

#######################################################################
function ml_curs_null()
#######################################################################
#  this function sets all of the elements in the curs
#  record to null
#
    initialize curs.* to null
end function
# ml_curs_null()

#######################################################################
function ml_next_null()
#######################################################################
#  this function sets all of the elements in the curs_next
#  record to null
#
    initialize curs_next.* to null
end function
# ml_next_null()

#######################################################################
function ml_curs_prep()
#######################################################################
#  this function transfers data from the curs_next record to the
#  curs record.
#
    let curs.doc_no = curs_next.doc_no
    let curs.incr_with_crdt = curs_next.incr_with_crdt
    let curs.debit_credit = curs_next.debit_credit
    let curs.amount = curs_next.amount
    let curs.acct_no = curs_next.acct_no
    let curs.department = curs_next.department
    let curs.acct_desc = curs_next.acct_desc
    let curs.doc_date = curs_next.doc_date
    let curs.inv_chk_no = curs_next.inv_chk_no
    let curs.orig_journal = curs_next.orig_journal
    let curs.ref_code = curs_next.ref_code
    let curs.doc_desc = curs_next.doc_desc
    let curs.reconciled = curs_next.reconciled 
end function
# ml_curs_prep()

#######################################################################
function ml_before_break()
#######################################################################
#  this function calls before_group() when appropriate.
#
    define
        no_break    smallint    # set to false if control break
    let no_break = true
    if no_break and curs.acct_no = curs_next.acct_no
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("acct_no")
        let no_break = false
    end if
    if no_break and curs.department = curs_next.department
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("department")
        let no_break = false
    end if
    if no_break and curs.doc_date = curs_next.doc_date
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("doc_date")
        let no_break = false
    end if
    if no_break and curs.orig_journal = curs_next.orig_journal
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("orig_journal")
        let no_break = false
    end if
    if no_break and curs.inv_chk_no = curs_next.inv_chk_no
    then else
        if no_break then call ml_curs_prep() end if
        call before_group("inv_chk_no")
        let no_break = false
    end if
end function
# ml_before_break()

#######################################################################
function ml_after_break()
#######################################################################
#  this function calls after_group() when appropriate.
#
    define
        no_break    smallint    # set to false if control break
    let no_break = true
    if no_break and curs.acct_no = curs_next.acct_no
    then else
        call after_group("acct_no")
        let no_break = false
    end if
    if no_break and curs.department = curs_next.department
    then else
        call after_group("department")
        let no_break = false
    end if
    if no_break and curs.doc_date = curs_next.doc_date
    then else
        call after_group("doc_date")
        let no_break = false
    end if
    if no_break and curs.orig_journal = curs_next.orig_journal
    then else
        call after_group("orig_journal")
        let no_break = false
    end if
    if no_break and curs.inv_chk_no = curs_next.inv_chk_no
    then else
        call after_group("inv_chk_no")
        let no_break = false
    end if
end function
# ml_after_break()

#######################################################################
function ml_output()
#######################################################################
#
    output to report report1 (rpt.*)
    return
end function
# ml_output()

#######################################################################
function ml_defaults()
#######################################################################
#  this function sets the ct.* record values
#
    let ct.prcname = "Checking Account Reconciliation"
    let ct.rtmargn = "FourGen Modifiable Accounting"
    let ct.destin = "report.out"
    let ct.quiet = "1"
    let rpt.co_name = al_compn()
end function
# ml_defaults()

